VERSION 5.00
Begin VB.Form frmMain 
   BorderStyle     =   3  'Fixed Dialog
   Caption         =   "DirectX-Graphics: Lesson 01"
   ClientHeight    =   3195
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   4680
   ControlBox      =   0   'False
   LinkTopic       =   "Form1"
   MaxButton       =   0   'False
   MinButton       =   0   'False
   ScaleHeight     =   213
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   312
   ShowInTaskbar   =   0   'False
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'###############################
'
'           Title: DirectX-Graphics Lesson 05
'           Desc: Basic 3D Geometry in DirectXGraphics8
'           Written by: Jack Hoxley
'           Date: December 8/9th 2000
'           Contact: Jollyjeffers@Greenonions.netscapeonline.co.uk
'
'###############################

Dim Dx As DirectX8 'The master Object, everything comes from here
Dim D3D As Direct3D8 'This controls all things 3D
Dim D3DDevice As Direct3DDevice8 'This actually represents the hardware doing the rendering
Dim D3DX As D3DX8 '//A helper library
Dim VBuffer As Direct3DVertexBuffer8 '//Stores our Geometry....

Dim bRunning As Boolean 'Controls whether the program is running or not...

'//We're using 3D vertices, but we're not using Lighting just yet...
'   which means we need to specify the vertex colour
Private Type LITVERTEX
    x As Single
    y As Single
    z As Single
    color As Long
    Specular As Long
    tu As Single
    tv As Single
End Type

'//The Descriptor for this vertex format...
Const Lit_FVF = (D3DFVF_XYZ Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR Or D3DFVF_TEX1)

'//We're going to define a cube; this method is
'   a very inefficient way of generating a cube; we're using
'   36 vertices, methods later on will allow us to use 8...
Dim Cube(35) As LITVERTEX


Const pi As Single = 3.14159265358979   'can be worked out using (4*atn(1))

'#####################
'##  NEW STUFF - MATRICES ##
'####################
Dim matWorld As D3DMATRIX '//How the vertices are positioned
Dim matView As D3DMATRIX '//Where the camera is/where it's looking
Dim matProj As D3DMATRIX '//How the camera projects the 3D world onto the 2D screen

'########################
'## FRAME RATE CALCULATIONS ##
'#######################
Private Declare Function GetTickCount Lib "kernel32" () As Long
Dim FPS_LastCheck As Long
Dim FPS_Count As Long
Dim FPS_Highest As Integer



'// Initialise : This procedure kick starts the whole process.
'// It'll return true for success, false if there was an error.
Public Function Initialise() As Boolean
On Error GoTo ErrHandler:

Dim DispMode As D3DDISPLAYMODE '//Describes our Display Mode
Dim D3DWindow As D3DPRESENT_PARAMETERS '//Describes our Viewport

Set Dx = New DirectX8  '//Create our Master Object
Set D3D = Dx.Direct3DCreate() '//Make our Master Object create the Direct3D Interface
Set D3DX = New D3DX8 '//Create our helper library...

'//We're going to use Fullscreen mode because I prefer it to windowed mode :)

'DispMode.Format = D3DFMT_X8R8G8B8
DispMode.Format = D3DFMT_R5G6B5 'If this mode doesn't work try the commented one above...
DispMode.Width = 640
DispMode.Height = 480

D3DWindow.SwapEffect = D3DSWAPEFFECT_FLIP
D3DWindow.BackBufferCount = 1 '//1 backbuffer only
D3DWindow.BackBufferFormat = DispMode.Format 'What we specified earlier
D3DWindow.BackBufferWidth = 640
D3DWindow.BackBufferHeight = 480
D3DWindow.hDeviceWindow = frmMain.hWnd
D3DWindow.EnableAutoDepthStencil = 1

If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D16) = D3D_OK Then
    'We can use a 16 bit Z-Buffer
    D3DWindow.AutoDepthStencilFormat = D3DFMT_D16 '//16 bit Z-Buffer
Else
    'We could now check for different modes available...
End If

'## ENUMERATE Z-BUFFERS ##
'This isn't really relevent, just check you're immediate window after execution has finished to
'see which depths you're hardware supports....
                    If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D16) = D3D_OK Then
                        Debug.Print "16 bit Z-Buffers are supported (D3DFMT_D16)"
                    End If
                    
                    If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D16_LOCKABLE) = D3D_OK Then
                        Debug.Print "Lockable 16 bit Z-Buffers are supported (D3DFMT_D16_LOCKABLE)"
                    End If
                    
                    If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D24S8) = D3D_OK Then
                        Debug.Print "32 bit divided between 24 bit Depth and 8 bit stencil are supported (D3DFMT_D24S8)"
                    End If
                    
                    If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D24X4S4) = D3D_OK Then
                        Debug.Print "32 bit divided between 24 bit depth, 4 bit stencil and 4 bit unused (D3DFMT_D24X4S4)"
                    End If
                    
                    If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D24X8) = D3D_OK Then
                        Debug.Print "24 bit Z-Buffer supported (D3DFMT_D24X8)"
                    End If
                    
                    If D3D.CheckDeviceFormat(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, DispMode.Format, D3DUSAGE_DEPTHSTENCIL, D3DRTYPE_SURFACE, D3DFMT_D32) = D3D_OK Then
                        Debug.Print "Pure 32 bit Z-buffer supported (D3DFMT_D32)"
                    End If

'//This line creates a device that uses a hardware device if possible; software vertex processing and uses the form as it's target
'//See the lesson text for more information on this line...
Set D3DDevice = D3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, _
                                                        D3DWindow)

'//Set the vertex shader to use our vertex format
D3DDevice.SetVertexShader Lit_FVF

'//Transformed and lit vertices dont need lighting
'   so we disable it...
D3DDevice.SetRenderState D3DRS_LIGHTING, False

'//Direct3D wont draw any triangles that are facing away from us;
'   to stop it doing this we stop it culling - it's good practise to design
'   your geometry so that you DONT need this option...
'   To do this, generate your vertices in a clockwise order... (by default it removes all CCW ordered triangles)
D3DDevice.SetRenderState D3DRS_CULLMODE, D3DCULL_NONE

'//We need to enable our Z Buffer
D3DDevice.SetRenderState D3DRS_ZENABLE, 1

'#####################
'##  NEW STUFF - MATRICES ##
'####################

'//1. The World Matrix
'       This is applied to all vertices that are sent to be rendered
'       Values in here can alter the position of a vertex; common
'       things are rotation, Translation and scaling.

D3DXMatrixIdentity matWorld '//Make an identity matrix
            'the identity matrix, when applied to a vertex will not alter
            'it in any way - useful for making a fresh start on a persistant variable
D3DDevice.SetTransform D3DTS_WORLD, matWorld 'commit this matrix to the device


'//2. The View Matrix
'       This can be thought of as the camera; defined by a start point
'       and an end point, as well as an up vector. The up vector usually
'       points upwards along the Y axis (so +Y becomes up and -Y becomes down)
'       Should you wish Z to be the vertical axis you need to alter this vector

D3DXMatrixLookAtLH matView, MakeVector(0, 5, 9), MakeVector(0, 0, 0), MakeVector(0, 1, 0)
        'Make the camera be at [0,4,0] and looking at the origin [0,0,0]
D3DDevice.SetTransform D3DTS_VIEW, matView



'//3. The projection Matrix
'       Once set up this can usually be left along; it defines what area of world-space
'       is visible to be rendered - the nearest and farthest reaches. It also specifies
'       how much we can see horizontally - a wide angle means we can see lots (like a landscape)
'       a small angle appears very zoomed in.
D3DXMatrixPerspectiveFovLH matProj, pi / 4, 1, 0.1, 500
    'We're going to use a pi/4 view angle (it's in radians) - pi/2 or pi/3 are also used
    'We're giving it an aspect ration of 1:1
    'and we're telling it to only draw triangles greater 0.1 from the camera
    'and less than 500 from the camera (in meters)
D3DDevice.SetTransform D3DTS_PROJECTION, matProj


'//We can only continue if Initialise Geometry succeeds;
'   If it doesn't we'll fail this call as well...
If InitialiseGeometry() = True Then
    Initialise = True '//We succeeded
    Exit Function
End If


ErrHandler:
'//We failed; for now we wont worry about why.
Debug.Print "Error Number Returned: " & Err.Number
Initialise = False
End Function

Public Sub Render()
'//1. We need to clear the render device before we can draw anything
'       This must always happen before you start rendering stuff...
D3DDevice.Clear 0, ByVal 0, D3DCLEAR_TARGET Or D3DCLEAR_ZBUFFER, 0, 1#, 0  '//Clear the screen black

'//2. Rendering the graphics...

D3DDevice.BeginScene
    'All rendering calls go between these two lines
    
    '//Draw the cube
    D3DDevice.SetStreamSource 0, VBuffer, Len(Cube(0))
    D3DDevice.DrawPrimitive D3DPT_TRIANGLELIST, 0, 12

    
D3DDevice.EndScene

'//3. Update the frame to the screen...
'       This is the same as the Primary.Flip method as used in DirectX 7
'       These values below should work for almost all cases...
D3DDevice.Present ByVal 0, ByVal 0, 0, ByVal 0
End Sub

Private Sub Form_Click()
bRunning = False
End Sub

Private Sub Form_Load()
Dim RotateAngle As Single
Dim matTemp As D3DMATRIX '//To hold temporary stuff...

Me.Show '//Make sure our window is visible

bRunning = Initialise()
Debug.Print "Device Creation Return Code : ", bRunning 'So you can see what happens...

Do While bRunning
    'We're going to update our world matrix so that the object does
    'something vaguely interesting... :)
    
    RotateAngle = RotateAngle + 0.1 'alter this if it appears to go stupidly fast... who need artificial motion blur... hehe
    If RotateAngle >= 360 Then RotateAngle = RotateAngle - 360
    
    D3DXMatrixIdentity matWorld '//Reset our world matrix
    
    D3DXMatrixIdentity matTemp
    D3DXMatrixRotationX matTemp, RotateAngle * (pi / 180)
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
    D3DXMatrixIdentity matTemp
    D3DXMatrixRotationZ matTemp, RotateAngle * (pi / 180)
    D3DXMatrixMultiply matWorld, matWorld, matTemp
    
       
    D3DDevice.SetTransform D3DTS_WORLD, matWorld

    Render '//Update the frame...
    
    '//Frame rate calculations
    If GetTickCount() - FPS_LastCheck >= 1000 Then
        If FPS_Count >= FPS_Highest Then FPS_Highest = FPS_Count
        FPS_Count = 0 'reset the counter
        FPS_LastCheck = GetTickCount()
    End If
    FPS_Count = FPS_Count + 1
    
    DoEvents '//Allow windows time to think; otherwise you'll get into a really tight (and bad) loop...
    
Loop '//Begin the next frame...

'//If we've gotten to this point the loop must have been terminated
'   So we need to clean up after ourselves. This isn't essential, but it's
'   good coding practise.

On Error Resume Next 'If the objects were never created;
'                               (the initialisation failed) we might get an
'                               error when freeing them... which we need to
'                               handle, but as we're closing anyway...
Set D3DDevice = Nothing
Set D3D = Nothing
Set Dx = Nothing
Debug.Print "All Objects Destroyed"

'//Around abouts 2000-3000 on a good computer (3824 on my 700Mhz Athlon/GeForce 256 DDR)
Debug.Print "Highest frame rate achieved whilst running this demo was: " & FPS_Highest & " frames per second"

'//Final termination:
Unload Me
End
End Sub



Private Function InitialiseGeometry() As Boolean
    
    
On Error GoTo BailOut: '//Setup our Error handler

'//I used the Immediate window to
'   get the RGB() values for each of these...
Const C000 As Long = 255     '//Red
Const C001 As Long = 65280  '//Green
Const C100 As Long = 16711680   '//Blue
Const C101 As Long = 16711935 '//Magenta
Const C010 As Long = 65535 '//Yellow
Const C011 As Long = 16776960 '//Cyan
Const C110 As Long = 16777215 '//White
Const C111 As Long = 8421631 '//Orange

Const DFC As Single = 1.5  '//Something I added a little after writing the tutorial;
                                                            'Using a value greater than 1 will allow you to see each face quite easily
                                                            'and a value of 1 will create a perfect cube...

'//Note that we are now using Vertex buffers to
'   store the vertex data...

'//1. Fill the structures with the data

        'Front
        Cube(0) = CreateLitVertex(-1, 1, DFC, C011, 0, 0, 0)
        Cube(1) = CreateLitVertex(1, 1, DFC, C111, 0, 0, 0)
        Cube(2) = CreateLitVertex(-1, -1, DFC, C001, 0, 0, 0)
        Cube(3) = CreateLitVertex(1, 1, DFC, C111, 0, 0, 0)
        Cube(4) = CreateLitVertex(-1, -1, DFC, C001, 0, 0, 0)
        Cube(5) = CreateLitVertex(1, -1, DFC, C101, 0, 0, 0)
        
        'Back
        Cube(6) = CreateLitVertex(-1, 1, -DFC, C010, 0, 0, 0)
        Cube(7) = CreateLitVertex(1, 1, -DFC, C110, 0, 0, 0)
        Cube(8) = CreateLitVertex(-1, -1, -DFC, C000, 0, 0, 0)
        Cube(9) = CreateLitVertex(1, 1, -DFC, C110, 0, 0, 0)
        Cube(10) = CreateLitVertex(-1, -1, -DFC, C000, 0, 0, 0)
        Cube(11) = CreateLitVertex(1, -1, -DFC, C100, 0, 0, 0)
        
        'Right
        Cube(12) = CreateLitVertex(-DFC, 1, -1, C010, 0, 0, 0)
        Cube(13) = CreateLitVertex(-DFC, 1, 1, C011, 0, 0, 0)
        Cube(14) = CreateLitVertex(-DFC, -1, -1, C000, 0, 0, 0)
        Cube(15) = CreateLitVertex(-DFC, 1, 1, C011, 0, 0, 0)
        Cube(16) = CreateLitVertex(-DFC, -1, -1, C000, 0, 0, 0)
        Cube(17) = CreateLitVertex(-DFC, -1, 1, C001, 0, 0, 0)
        
        'Left
        Cube(18) = CreateLitVertex(DFC, 1, -1, C110, 0, 0, 0)
        Cube(19) = CreateLitVertex(DFC, 1, 1, C111, 0, 0, 0)
        Cube(20) = CreateLitVertex(DFC, -1, -1, C100, 0, 0, 0)
        Cube(21) = CreateLitVertex(DFC, 1, 1, C111, 0, 0, 0)
        Cube(22) = CreateLitVertex(DFC, -1, -1, C100, 0, 0, 0)
        Cube(23) = CreateLitVertex(DFC, -1, 1, C101, 0, 0, 0)
        
        'Top
        Cube(24) = CreateLitVertex(-1, DFC, 1, C011, 0, 0, 0)
        Cube(25) = CreateLitVertex(1, DFC, 1, C111, 0, 0, 0)
        Cube(26) = CreateLitVertex(-1, DFC, -1, C010, 0, 0, 0)
        Cube(27) = CreateLitVertex(1, DFC, 1, C111, 0, 0, 0)
        Cube(28) = CreateLitVertex(-1, DFC, -1, C010, 0, 0, 0)
        Cube(29) = CreateLitVertex(1, DFC, -1, C110, 0, 0, 0)
        
        'Top
        Cube(30) = CreateLitVertex(-1, -DFC, 1, C001, 0, 0, 0)
        Cube(31) = CreateLitVertex(1, -DFC, 1, C101, 0, 0, 0)
        Cube(32) = CreateLitVertex(-1, -DFC, -1, C000, 0, 0, 0)
        Cube(33) = CreateLitVertex(1, -DFC, 1, C101, 0, 0, 0)
        Cube(34) = CreateLitVertex(-1, -DFC, -1, C000, 0, 0, 0)
        Cube(35) = CreateLitVertex(1, -DFC, -1, C100, 0, 0, 0)


'//2. Create us a blank vertex buffer of the required size

            Set VBuffer = D3DDevice.CreateVertexBuffer(Len(Cube(0)) * 36, 0, Lit_FVF, D3DPOOL_DEFAULT)
            If VBuffer Is Nothing Then Exit Function '//Error handler



'//3. Fill the created vertex buffer with the data
            D3DVertexBuffer8SetData VBuffer, 0, Len(Cube(0)) * 36, 0, Cube(0)

InitialiseGeometry = True
Exit Function
BailOut:
InitialiseGeometry = False
End Function

'//Another wrapper function similiar to that used to generate transformed and lit vertices in
'   earlier lessons.
Private Function CreateLitVertex(x As Single, y As Single, z As Single, Diffuse As Long, Specular As Long, tu As Single, tv As Single) As LITVERTEX
        CreateLitVertex.x = x
        CreateLitVertex.y = y
        CreateLitVertex.z = z
        CreateLitVertex.color = Diffuse
        CreateLitVertex.Specular = Specular
        CreateLitVertex.tu = tu
        CreateLitVertex.tv = tv
End Function

'//A new wrapper function - so we can create vectors inline with function calls.
Private Function MakeVector(x As Single, y As Single, z As Single) As D3DVECTOR
    MakeVector.x = x
    MakeVector.y = y
    MakeVector.z = z
End Function
